/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 10 janvier 2007
   derniere modification le 25/06/2007 */

#ifndef __SCECBUFFERS_H__
#define __SCECBUFFERS_H__


/* constantes de definition des types de sommets. (v_type)
   la constante SCE_POSITION peut valoire n'importe quelle valeur entiere
   positive, les autre doivent imperativement valloires PREC+1
   ou PREC represente la constante precedemment definie. */
#define SCE_POSITION 1
#define SCE_COLOR 2
#define SCE_NORMAL 3
/* coordonnees de texture */
#define SCE_TEXCOORD0 4
#define SCE_TEXCOORD1 5
#define SCE_TEXCOORD2 6
#define SCE_TEXCOORD3 7
#define SCE_TEXCOORD4 8
#define SCE_TEXCOORD5 9
#define SCE_TEXCOORD6 10
#define SCE_TEXCOORD7 11
/* s'etend jusqu'au maximum supporte par ma machine, cf CBuffers.c */
/* attributs de sommet */
#define SCE_ATTRIB0 100
#define SCE_ATTRIB1 101
#define SCE_ATTRIB2 102
#define SCE_ATTRIB3 103
#define SCE_ATTRIB4 104
#define SCE_ATTRIB5 105
#define SCE_ATTRIB6 106
#define SCE_ATTRIB7 107
#define SCE_ATTRIB8 108
#define SCE_ATTRIB9 109
#define SCE_ATTRIB10 110
#define SCE_ATTRIB11 111
#define SCE_ATTRIB12 112
#define SCE_ATTRIB13 113
#define SCE_ATTRIB14 114
#define SCE_ATTRIB15 115
/* s'etend jusqu'au maximum supporte par ma machine, cf CBuffers.c */


/* declaration d'un type de sommet
   divers information sur la position des donnees dans l'objet tampon opengl */
typedef struct sce_cvertexdeclaration SCE_CVertexDeclaration;
struct sce_cvertexdeclaration
{
    /* type du sommet (vertex, normal, ...) */
    unsigned int v_type;
    
    /* type de variables (int, float, ...) */
    SCEenum type;
    
    /* taille du groupe de donnees (d'un vecteur)
       2 = 2*v_type :
       si v_type = SCE_POSITION, les coordonnees sont envoyees en 2D */
    GLint n_vars;
    
    /* position du premier octet de donnes (byte) */
    GLint first;
    
    /* marge entre chaque groupes (vecteur) de donnees (en bytes) */
    GLsizei stride;
};


/* donnees tampon
   enregistre une sequence de donnees */
typedef struct sce_cbufferdata SCE_CBufferData;
struct sce_cbufferdata
{
    /* pointeur sur les donnees */
    void *data;
    
    /* taille reservee pour le pointeur de donnees */
    size_t data_size;
    
    /* premiere occurence des donnees dans le tampon
       auquel elles appartiennent */
    size_t first;
    
    /* autorisation de supprimmer les donnees avec SCE_free() */
    int free_data;
};


/* objet tampon de sommets opengl */
typedef struct sce_cvertexbuffer SCE_CVertexBuffer;
struct sce_cvertexbuffer
{
    /* id opengl */
    GLuint id;
    
    /* nombre de donnees */
    unsigned int n_data;
    
    /* donnees du tampon */
    SCE_CBufferData *datas;
    
    /* taille additionnee des precedents stockage de donnees
       au final, size = la taille total du tampon */
    size_t size;
    
    /* nombre de declarations de sommets
       une declaration = pour un type de sommet
       donc n_decs = nombre de types de sommet */
    unsigned int n_decs;
    
    /* declaration des sommets */
    SCE_CVertexDeclaration *dec;
};


/* objet tampon d'indices opengl */
typedef struct sce_cindexbuffer SCE_CIndexBuffer;
struct sce_cindexbuffer
{
    /* id opengl */
    GLuint id;
    
    /* nombre de donnees */
    unsigned int n_data;
    
    /* donnees du tampon */
    SCE_CBufferData *datas;
    
    /* taille additionnee des precedents stockage de donnees
       au final, size = la taille total du tampon */
    size_t size;
};




/* cree un tampon de sommets */
SCE_CVertexBuffer* SCE_CCreateVertexBuffer(void);

/* cree un tampon d'indices */
SCE_CIndexBuffer* SCE_CCreateIndexBuffer(void);



/* envoie des donnees a un tampon de sommets */
int SCE_CSetVertexBufferData(SCE_CVertexBuffer*, int, size_t, void*);

/* envoie des donnees a un tampon d'indices */
int SCE_CSetIndexBufferData(SCE_CIndexBuffer*, int, size_t, void*);



/* renvoie la taille additionnee de toutes
   les donnees stockees dans le tampon de sommets.
   cette taille est egale au premier emplacement
   disponible pour stocker des donnees dans le tampon */
size_t SCE_CGetVertexBufferFirst(SCE_CVertexBuffer*);



/* envoie des declarations de vertex a un tampon de sommets */
void SCE_CSetVertexDeclaration(SCE_CVertexBuffer*, unsigned int,
                               SCE_CVertexDeclaration*);



/* construit un tampon de sommets, envoie les donnees a opengl */
void SCE_CBuildVertexBuffer(SCE_CVertexBuffer*, SCEenum);

/* construit un tampon d'indices, envoie les donnees a opengl */
void SCE_CBuildIndexBuffer(SCE_CIndexBuffer*, SCEenum);



/* verouille un tampon de sommets
   et renvoie l'adresse memoire de la zone de stockage du tampon */
void* SCE_CLockVertexBuffer(SCE_CVertexBuffer*, SCEenum);

/* deverouille un tampon de sommets
   rend invalide le pointeur renvoye par LockBuffer */
void SCE_CUnlockVertexBuffer(SCE_CVertexBuffer*);



/* verouille un tampon de d'indices
   et renvoie l'adresse memoire de la zone de stockage du tampon */
void* SCE_CLockIndexBuffer(SCE_CIndexBuffer*, SCEenum);

/* deverouille un tampon de sommets
   rend invalide le pointeur renvoye par LockBuffer */
void SCE_CUnlockIndexBuffer(SCE_CIndexBuffer*);



/* met a jour les donnees contenues dans un tampon de sommets */
int SCE_CUpdateVertexBuffer(SCE_CVertexBuffer*);

/* met a jour les donnes contenues dans un tampon d'indices */
int SCE_CUpdateIndexBuffer(SCE_CIndexBuffer*);



/* selectionne un tampon de sommets
   et le defini comme tampon de sommets actif pour le rendu */
void SCE_CUseVertexBuffer(SCE_CVertexBuffer*);

/* selectionne un tampon d'indices
   et le defini comme tampon d'indices actif a utiliser pour le rendu */
void SCE_CUseIndexBuffer(SCE_CIndexBuffer*);



/* dessine le(s) tampon(s) actif(s) selectionnes pour le rendu */
void SCE_CDrawBuffer(SCEenum, GLint, GLsizei);

/* dessine le(s) tampon(s) actif(s) selectionnes pour le rendu 
   en utilisant le tampon actif d'indices*/
void SCE_CDrawIndexedBuffer(SCEenum, size_t, SCEenum, GLsizei);



/* vide la memoire allouee pour un tampon de sommets */
void SCE_CDeleteVertexBuffer(SCE_CVertexBuffer*);

/* vide la memoire allouee pour un tampon d'indices */
void SCE_CDeleteIndexBuffer(SCE_CIndexBuffer*);


#endif /* guard */
