/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 11 mars 2007
   derniere modification le 04/07/2007 */


#ifndef __SCETEXTURE_H__
#define __SCETEXTURE_H__


/* constantes propres a Texture */
/* filtres de redimensionnement */ /* NOTE: noms constantes a revoir (conflit avec SCE_TEX_NEAREST & cie) */
#define SCE_FILTER_NEAREST      ILU_NEAREST
#define SCE_FILTER_LINEAR       ILU_LINEAR
#define SCE_FILTER_BILINEAR     ILU_BILINEAR
#define SCE_FILTER_BOX          ILU_SCALE_BOX
#define SCE_FILTER_TRIANGLE     ILU_SCALE_TRIANGLE
#define SCE_FILTER_BELL         ILU_SCALE_BELL
#define SCE_FILTER_BSPLINE      ILU_SCALE_BSPLINE
#define SCE_FILTER_LANCZOS3     ILU_SCALE_LANCZOS3
#define SCE_FILTER_MITCHELL     ILU_SCALE_MITCHELL

/* types supplementaires acceptables pour la creation d'une texture */
/* NOTE caca : ces constantes doivent toutes etre differentes
               de celles de CFramebuffer... */
#define SCE_RENDER_COLOR 10
#define SCE_RENDER_COLOR_CUBE 11
#define SCE_RENDER_DEPTH 12
#define SCE_RENDER_DEPTH_CUBE 13

/* defini la face de la cubemap sur laquelle rendre */
#define SCE_RENDER_POSX 0
#define SCE_RENDER_NEGX 1
#define SCE_RENDER_POSY 2
#define SCE_RENDER_NEGY 3
#define SCE_RENDER_POSZ 4
#define SCE_RENDER_NEGZ 5



/* structure d'une image */
typedef struct sce_stexdata SCE_STexData;
struct sce_stexdata
{
    ILuint id;              /* identifiant devil */
    
    unsigned int n_data;    /* nombre de donnees */ /* mipmap (1) */
    SCE_CTexData *cdata;    /* donnees cote coeur */
    
    int freetexels;         /* booleen, true: les texels contenus dans cdata
                                              doivent etre liberes (SCE_free) */
};

/* structure d'une texture, contenant son(ses) image(s) (cube map) */
typedef struct sce_stexture SCE_STexture;
struct sce_stexture
{
    SCE_CFramebuffer *fb[6];/* frame bufferS (pour les cubemaps) */
    
    SCE_CTexture *tex;      /* texture du "coeur" */
    
    unsigned int n_data;    /* nombre de donnees */ /* cube map (1) */
    SCE_STexData **data;    /* donnees */
    
    int loaded;             /* booleens parlant d'eux-meme */
    int builded;
};

/* (1)*: plus pratique au niveau des loaders.
         une image d'une face du cube peut etre constituee de plusieurs
         niveaux de mipmap, alors qu'un niveau de mipmap ne peux pas
         representer plusieurs faces de la cubemap a la fois */


/* renvoie d'identifiant des textures pour le media manager */
int SCE_Texture_MediaTypeID(void);

/* initialise le gestionnaire de textures */
int SCE_Texture_Init(void);

/* quitte le gestionnaire de textures */
int SCE_Texture_Quit(void);


/* cree une texture en prenant son type */
SCE_STexture* SCE_Texture_Create(SCEenum, int, int);

/* supprime une texture */
void SCE_Texture_Delete(SCE_STexture*);

#if 0
/* recherche le type (enumerateur IL) d'une image d'apres son extension */
SCEenum SCE_Texture_GetImageType(const char*);
#endif

/* defini le filtre a appliquer a l'image lors du redimensionnement */
void SCE_Texture_SetResizeFilter(SCEenum);

/* defini si la texture sera agrandie ou retrecie lors du redimensionnement */
void SCE_Texture_ReduceImages(int);

/* redimensionne une texture */
void SCE_Texture_Resize(SCE_STexture*, unsigned int, int, int, int);

/* fonctions de chargement d'images */
/* fonction de chargement des fichiers en utilisant DevIL */
void* SCE_Texture_DevILLoader(FILE*, const char*, void*);
/* chargement des images au format DDS */
void* SCE_Texture_DDSLoader(FILE*, const char*, void*);


/* ajoute des donnees a une texture */
int SCE_Texture_AddData(SCE_STexture*, SCEenum, SCE_STexData*);

/* ajoute une image a une texture */
int SCE_Texture_AddImage(SCE_STexture*, SCEenum, const char*);


/* construit la texture */
int SCE_Texture_Build(SCE_STexture*, int);

/* met a jour les donnees de la texture */
int SCE_Texture_Update(SCE_STexture*);


/* cree et charge une texture directement a partir d'un nom de fichier */
SCE_STexture* SCE_Texture_CreateFromFile(const char*, int);


/* defini si la modification de texture via Use est possible */
void SCE_Texture_Active(int);
/* version "verbose" */
void SCE_Texture_Enable(void);
void SCE_Texture_Disable(void);

/* defini la texture active et son unite de texture */
void SCE_Texture_Use(SCE_STexture*, unsigned int);

/* defini la texture dans laquel effectuer le rendu
   (sous condition que la texture soit de type "render texture") */
void SCE_Texture_RenderTo(SCE_STexture*, SCEuint);


#endif /* guard */
